/*
 * path.h - Toolpath operations
 *
 * Written 2010-2011 by Werner Almesberger
 * Copyright 2010-2011 Werner Almesberger
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */


#ifndef PATH_H
#define PATH_H


struct point {
	double x, y, z;	/* mm */
	struct point *next;
};

struct path {
	struct point *first, *last;
	double r_tool;	/* mm */
	int outside;	/* non-zero to mark path as an outside edge */
	int notch;	/* non-zero to enable dog-boning for path */
	const char *id;	/* identifier assigned by generator */
	struct path *next;
};


struct path *path_new(double r_tool, const char *id);
void path_add(struct path *path, double x, double y, double z);
struct path *path_reverse(const struct path *path);
int path_tool_is_left(const struct path *path);
struct path *path_offset(const struct path *path, int left, int notch);
const struct path *path_find_leftmost(const struct path *path);
void path_free(struct path *path);
struct path *path_connect(struct path *path);
void path_stats(const struct path *path);

#endif /* !PATH_H */
